import 'dart:async';
import 'package:flutter/services.dart';

class _Methods {
  static const start = 'start';
  static const stop = 'stop';
  static const outputFrame = 'outputFrame';
}

class _Strings {
  static const inputChannelName = 'com.wearemobilefirst.audio_io.inputAudio';
  static const methodChannelName = 'com.wearemobilefirst.audio_io';
}

class AudioIo {
  static const MethodChannel _methods = const MethodChannel(_Strings.methodChannelName);
  static const EventChannel _input = const EventChannel(_Strings.inputChannelName);
  static StreamSubscription<List<double>> _outputSubscription;
  static StreamSubscription _inputSubscription;
  static AudioIo instance = AudioIo();

  StreamController<List<double>> _outputController = StreamController<List<double>>.broadcast();
  StreamController<List<double>> _inputController = StreamController<List<double>>.broadcast();

  Stream<List<double>> get input {
    return _inputController.stream;
  }

  Sink<List<double>> get output {
    return _outputController.sink;
  }

  Future<void> start() async {
    _outputSubscription?.cancel();
    _inputSubscription?.cancel();
    _outputSubscription = _outputController.stream.listen((output){
        _methods.invokeMethod(_Methods.outputFrame, output);
    });
    _inputSubscription = _input.receiveBroadcastStream().listen((data){
      final listData = data as List;
      final converted = listData?.map((element){
        return element as double;
      })?.toList();
      _inputController.sink.add(converted);
    });
    await _methods.invokeMethod(_Methods.start);
  }

  Future<void> stop() async {
    await _methods.invokeMethod(_Methods.stop);
  }

  void dispose(){
    _outputController.sink.close();
    _outputController.close();
    _inputController.sink.close();
    _inputController.close();
  }

}
